@@ -2,7 +2,7 @@
  * Graphics - sprites management
 
  * Copyright (C) 1997, 1998, 1999, 2002, 2003  Seth A. Robinson
- * Copyright (C) 2007, 2008  Sylvain Beucler
+ * Copyright (C) 2007, 2008, 2009  Sylvain Beucler
 
  * This file is part of GNU FreeDink
 
@@ -21,8 +21,14 @@
  * <http://www.gnu.org/licenses/>.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gfx_sprites.h"
 
+#include <stdlib.h>
+#include <string.h>
 #include "SDL_image.h"
 
 #include "gfx.h"
@@ -39,6 +45,12 @@
   seq[], GFX_k[], k[], no_running_main
 */
 
+/* TODO: get rid of either k or GFX_k */
+struct pic_info     k[MAX_SPRITES];       // Sprite data
+struct GFX_pic_info GFX_k[MAX_SPRITES];   // Sprite data (SDL)
+
+struct sequence seq[MAX_SEQUENCES];
+
 
 static int please_wait = 0;
 
@@ -46,17 +58,24 @@ static int please_wait = 0;
  * Free memory used by sprites. It's not much useful in itself, since
  * it's only called when we're exiting the game, but it does avoid
  * memory leak warnings when FreeDink is analyzed by Valgrind or other
- * memory checkers.
+ * memory checkers. It also supports loading and unloading the
+ * subsystem several times.
  */
 void sprites_unload(void)
 {
   int i = 0;
   for (i = 0; i < MAX_SPRITES; i++)
-    if (GFX_k[i].k != NULL)
-      SDL_FreeSurface(GFX_k[i].k);
+    {
+      if (GFX_k[i].k != NULL)
+	SDL_FreeSurface(GFX_k[i].k);
+      GFX_k[i].k = NULL;
+    }
   for (i = 0; i < MAX_SEQUENCES; i++)
-    if (seq[i].ini != NULL)
-      free(seq[i].ini);
+    {
+      if (seq[i].ini != NULL)
+	free(seq[i].ini);
+      seq[i].ini = NULL;
+    }
 }
 
 
@@ -94,9 +113,10 @@ static void draw_wait()
 static int next_slot()
 {
   /* Start index at 1 instead of 0. A few parts in the game rely on
-     this (e.g. if getpic(...) < 1). I noticed that sprite in slot 0
-     (by default, this would be a small white square) would be
-     displayed temporarily on the screen in some situations . */
+     this (e.g. if getpic(...) < 1). */
+  /* TODO: I noticed that sprite in slot 0 (by default, this would be
+     a small white square) would be displayed temporarily on the
+     screen in some situations.. */
   int i = 1;
   while (i < MAX_SPRITES && GFX_k[i].k != NULL)
     i++;
@@ -112,13 +132,16 @@ static void free_seq(int seq_no)
 {
   int i = 1;
   int slot_index = -1;
-  while ((slot_index = seq[seq_no].frame[i]) != 0)
+  while (i < MAX_FRAMES_PER_ABUSED_SEQUENCE+1
+	 && (slot_index = seq[seq_no].frame[i]) != 0)
     {
       SDL_FreeSurface(GFX_k[slot_index].k);
       GFX_k[slot_index].k = NULL;
       i++;
     }
   /* 0 means end-of-sequence, no more frames */
+  if (i == MAX_FRAMES_PER_ABUSED_SEQUENCE+1)
+    log_error("Invalid sequence %d, just avoided a buffer overflow\n", seq_no);
 }
 
 
@@ -154,13 +177,13 @@ void load_sprite_pak(char seq_path_prefix[100], int seq_no, int delay, int xoffs
 
   if (!FastFileInit(fullpath, 5))
     {
-      Msg("Could not load dir.ff art file %s", crap);
+      log_error("Could not load dir.ff art file %s", crap);
       free(fullpath);
       free(seq_dirname);
       return;
     }
-  free(fullpath);
   free(seq_dirname);
+  /* keep 'fullpath' for error messages, free() it later */
 
 
   int oo;
@@ -169,8 +192,8 @@ void load_sprite_pak(char seq_path_prefix[100], int seq_no, int delay, int xoffs
       int myslot = next_slot();
       if (myslot >= MAX_SPRITES)
 	{
-	  fprintf(stderr, "No sprite slot available! Index %d out of %d.\n",
-		  myslot, MAX_SPRITES);
+	  log_error("No sprite slot available! Index %d out of %d.",
+		    myslot, MAX_SPRITES);
 	  break;
 	}
 
@@ -182,30 +205,37 @@ void load_sprite_pak(char seq_path_prefix[100], int seq_no, int delay, int xoffs
       HFASTFILE pfile = FastFileOpen(crap);
 
       if (pfile == NULL)
+	/* File not present in this fastfile - either missing file or
+	   end of sequence */
 	break;
       
       // GFX
-      Uint8 *buffer;
-      SDL_RWops *rw;
-      buffer = (Uint8 *) FastFileLock (pfile, 0, 0);
-      if (buffer != NULL)
+      SDL_RWops *rw = FastFileLock(pfile);
+      if (rw == NULL)
+	{
+	  /* rwops error? */
+	  log_error("Failed to open '%s' in fastfile '%s'", crap, fullpath);
+	}
+      else
 	{
-	  rw = SDL_RWFromMem (buffer, FastFileLen (pfile));
 	  /* We use IMG_Load_RW instead of SDL_LoadBMP because there
-	     is no _RW access in plain SDL. However there is not
+	     is no _RW access in plain SDL. However there is no
 	     intent to support anything else than 8bit BMPs. */
 	  GFX_k[myslot].k = IMG_Load_RW(rw, 1); // auto free()
+	  if (GFX_k[myslot].k == NULL)
+	    log_error("Failed to load %s from fastfile %s: %s", crap, fullpath, SDL_GetError());
 	}
       if (GFX_k[myslot].k == NULL)
 	{
-	  fprintf(stderr, "Failed to load %s from fastfile\n", crap);
+	  log_error("Failed to load %s from fastfile %s (see error above)", crap, fullpath);
 	  FastFileClose(pfile);
 	  break;
 	}
       if (GFX_k[myslot].k->format->BitsPerPixel != 8)
 	{
-	  fprintf(stderr, "Failed to load %s from fastfile:"
-		  " only 8bit paletted bitmaps are supported in dir.ff archives.\n", crap);
+	  log_error("Failed to load %s from fastfile %s:"
+		    " only 8bit paletted bitmaps are supported in dir.ff archives.",
+		    crap, fullpath);
 	  SDL_FreeSurface(GFX_k[myslot].k);
 	  continue;
 	}
@@ -370,7 +400,8 @@ void load_sprite_pak(char seq_path_prefix[100], int seq_no, int delay, int xoffs
   seq[seq_no].len = oo - 1;
   
   if (oo == 1)
-    fprintf(stderr, "Sprite_load_pak error:  Couldn't load %s.\n", crap);
+    log_error("Sprite_load_pak error:  Couldn't load %s in %s.", crap, fullpath);
+  free(fullpath);
 }
 
 
@@ -385,13 +416,11 @@ void load_sprites(char seq_path_prefix[100], int seq_no, int delay, int xoffset,
   char *fullpath = NULL;
   int use_fallback = 0;
 
-  int notanim = 0, black = 0, leftalign = 0;
+  int notanim = 0, black = 0;
   if ((flags & DINKINI_NOTANIM) == DINKINI_NOTANIM)
     notanim = 1;
   if ((flags & DINKINI_BLACK) == DINKINI_BLACK)
     black = 1;
-  if ((flags & DINKINI_LEFTALIGN) == DINKINI_LEFTALIGN)
-    leftalign = 1;
 
   if (no_running_main)
     draw_wait();
@@ -403,43 +432,52 @@ void load_sprites(char seq_path_prefix[100], int seq_no, int delay, int xoffset,
   /* - ../dink/.../dir.ff */
   /* - ../dink/.../...01.BMP */
   char *seq_dirname = pdirname(seq_path_prefix);
-  sprintf(crap, "%s/dir.ff", seq_dirname);
-  fullpath = paths_dmodfile(crap);
-  //Msg("Checking for %s..", crap);
-  if (exist(fullpath))
-    {
-      free(fullpath);
-      free(seq_dirname);
-      load_sprite_pak(seq_path_prefix, seq_no, delay, xoffset, yoffset,
-		      hardbox, flags, /*true*/1);
-      return;
-    }
-  free(fullpath);
-  
   int exists = 0;
-  sprintf(crap, "%s01.BMP",seq_path_prefix);
-  fullpath = paths_dmodfile(crap);
-  exists = exist(fullpath);
-  free(fullpath);
+
   if (!exists)
     {
-      sprintf(crap, "%s/dir.ff",  seq_dirname);
-      fullpath = paths_fallbackfile(crap);
-      //Msg("Checking for %s..", crap);
+      sprintf(crap, "%s/dir.ff", seq_dirname);
+      fullpath = paths_dmodfile(crap);
       exists = exist(fullpath);
       free(fullpath);
       if (exists)
 	{
-	  load_sprite_pak(seq_path_prefix, seq_no, delay, xoffset, yoffset,
-			  hardbox, flags, /*false*/0);
 	  free(seq_dirname);
+	  load_sprite_pak(seq_path_prefix, seq_no, delay, xoffset, yoffset,
+			  hardbox, flags, /*true*/1);
 	  return;
 	}
-      else
+    }
+  
+  if (!exists)
+    {
+      sprintf(crap, "%s01.BMP",seq_path_prefix);
+      fullpath = paths_dmodfile(crap);
+      exists = exist(fullpath);
+      free(fullpath);
+    }
+
+  if (!exists)
+    {
+      sprintf(crap, "%s/dir.ff",  seq_dirname);
+      fullpath = paths_fallbackfile(crap);
+      exists = exist(fullpath);
+      free(fullpath);
+      if (exists)
 	{
-	  use_fallback = 1;
+	  free(seq_dirname);
+	  load_sprite_pak(seq_path_prefix, seq_no, delay, xoffset, yoffset,
+			  hardbox, flags, /*false*/0);
+	  return;
 	}
     }
+
+  if (!exists)
+    {
+      /* Let's look for the BMP below */
+      use_fallback = 1;
+    }
+
   free(seq_dirname);
 
 
@@ -453,8 +491,8 @@ void load_sprites(char seq_path_prefix[100], int seq_no, int delay, int xoffset,
       int myslot = next_slot();
       if (myslot >= MAX_SPRITES)
 	{
-	  fprintf(stderr, "No sprite slot available! Index %d out of %d.\n",
-		  myslot, MAX_SPRITES);
+	  log_error("No sprite slot available! Index %d out of %d.",
+		    myslot, MAX_SPRITES);
 	  break;
 	}
 
@@ -580,8 +618,7 @@ void load_sprites(char seq_path_prefix[100], int seq_no, int delay, int xoffset,
   if (oo == 1)
     {
       /* First frame didn't load! */
-      fprintf(stderr, "load_sprites: couldn't open %s: %s\n", crap, SDL_GetError());
-      Msg("load_sprites:  Anim %s not found.",seq_path_prefix);
+      log_error("load_sprites: anim '%s' not found: couldn't open '%s'", seq_path_prefix, crap);
     }
 }
 
